# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
from ..cog import runtime as cogruntime


class Options:
    keep_time: bool
    include_vars: bool
    show_starred: bool
    show_recently_viewed: bool
    show_search: bool
    show_headings: bool
    max_items: int
    query: str
    tags: list[str]
    # folderId is deprecated, and migrated to folderUid on panel init
    folder_id: typing.Optional[int]
    folder_uid: typing.Optional[str]

    def __init__(self, keep_time: bool = False, include_vars: bool = False, show_starred: bool = True, show_recently_viewed: bool = False, show_search: bool = False, show_headings: bool = True, max_items: int = 10, query: str = "", tags: typing.Optional[list[str]] = None, folder_id: typing.Optional[int] = None, folder_uid: typing.Optional[str] = None):
        self.keep_time = keep_time
        self.include_vars = include_vars
        self.show_starred = show_starred
        self.show_recently_viewed = show_recently_viewed
        self.show_search = show_search
        self.show_headings = show_headings
        self.max_items = max_items
        self.query = query
        self.tags = tags if tags is not None else []
        self.folder_id = folder_id
        self.folder_uid = folder_uid

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "keepTime": self.keep_time,
            "includeVars": self.include_vars,
            "showStarred": self.show_starred,
            "showRecentlyViewed": self.show_recently_viewed,
            "showSearch": self.show_search,
            "showHeadings": self.show_headings,
            "maxItems": self.max_items,
            "query": self.query,
            "tags": self.tags,
        }
        if self.folder_id is not None:
            payload["folderId"] = self.folder_id
        if self.folder_uid is not None:
            payload["folderUID"] = self.folder_uid
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "keepTime" in data:
            args["keep_time"] = data["keepTime"]
        if "includeVars" in data:
            args["include_vars"] = data["includeVars"]
        if "showStarred" in data:
            args["show_starred"] = data["showStarred"]
        if "showRecentlyViewed" in data:
            args["show_recently_viewed"] = data["showRecentlyViewed"]
        if "showSearch" in data:
            args["show_search"] = data["showSearch"]
        if "showHeadings" in data:
            args["show_headings"] = data["showHeadings"]
        if "maxItems" in data:
            args["max_items"] = data["maxItems"]
        if "query" in data:
            args["query"] = data["query"]
        if "tags" in data:
            args["tags"] = data["tags"]
        if "folderId" in data:
            args["folder_id"] = data["folderId"]
        if "folderUID" in data:
            args["folder_uid"] = data["folderUID"]        

        return cls(**args)


def variant_config() -> cogruntime.PanelCfgConfig:
    return cogruntime.PanelCfgConfig(
        identifier="dashlist",
        options_from_json_hook=Options.from_json,
        field_config_from_json_hook=None,
    )




# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing


class PublicDashboard:
    # Unique public dashboard identifier
    uid: str
    # Dashboard unique identifier referenced by this public dashboard
    dashboard_uid: str
    # Unique public access token
    access_token: typing.Optional[str]
    # Flag that indicates if the public dashboard is enabled
    is_enabled: bool
    # Flag that indicates if annotations are enabled
    annotations_enabled: bool
    # Flag that indicates if the time range picker is enabled
    time_selection_enabled: bool

    def __init__(self, uid: str = "", dashboard_uid: str = "", access_token: typing.Optional[str] = None, is_enabled: bool = False, annotations_enabled: bool = False, time_selection_enabled: bool = False):
        self.uid = uid
        self.dashboard_uid = dashboard_uid
        self.access_token = access_token
        self.is_enabled = is_enabled
        self.annotations_enabled = annotations_enabled
        self.time_selection_enabled = time_selection_enabled

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "uid": self.uid,
            "dashboardUid": self.dashboard_uid,
            "isEnabled": self.is_enabled,
            "annotationsEnabled": self.annotations_enabled,
            "timeSelectionEnabled": self.time_selection_enabled,
        }
        if self.access_token is not None:
            payload["accessToken"] = self.access_token
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "uid" in data:
            args["uid"] = data["uid"]
        if "dashboardUid" in data:
            args["dashboard_uid"] = data["dashboardUid"]
        if "accessToken" in data:
            args["access_token"] = data["accessToken"]
        if "isEnabled" in data:
            args["is_enabled"] = data["isEnabled"]
        if "annotationsEnabled" in data:
            args["annotations_enabled"] = data["annotationsEnabled"]
        if "timeSelectionEnabled" in data:
            args["time_selection_enabled"] = data["timeSelectionEnabled"]        

        return cls(**args)



# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
from ..cog import builder as cogbuilder
from ..models import datasource
from ..models import dashboard


class Dataquery(cogbuilder.Builder[datasource.Dataquery]):
    _internal: datasource.Dataquery

    def __init__(self):
        self._internal = datasource.Dataquery()

    def build(self) -> datasource.Dataquery:
        """
        Builds the object.
        """
        return self._internal    
    
    def panel_id(self, panel_id: int) -> typing.Self:    
        """
        Panel ID from wich the queries will be reused.
        """
            
        self._internal.panel_id = panel_id
    
        return self
    
    def ref_id(self, ref_id: str) -> typing.Self:    
        """
        A unique identifier for the query within the list of targets.
        In server side expressions, the refId is used as a variable name to identify results.
        By default, the UI will assign A->Z; however setting meaningful names may be useful.
        """
            
        self._internal.ref_id = ref_id
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        """
        true if query is disabled (ie should not be returned to the dashboard)
        Note this does not always imply that the query should not be executed since
        the results from a hidden query may be used as the input to other queries (SSE etc)
        """
            
        self._internal.hide = hide
    
        return self
    
    def datasource(self, datasource: dashboard.DataSourceRef) -> typing.Self:    
        """
        For mixed data sources the selected datasource is on the query level.
        For non mixed scenarios this is undefined.
        TODO find a better way to do this ^ that's friendly to schema
        TODO this shouldn't be unknown but DataSourceRef | null
        """
            
        self._internal.datasource = datasource
    
        return self
    



# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing


class Preferences:
    """
    Spec defines user, team or org Grafana preferences
    swagger:model Preferences
    """

    # UID for the home dashboard
    home_dashboard_uid: typing.Optional[str]
    # The timezone selection
    # TODO: this should use the timezone defined in common
    timezone: typing.Optional[str]
    # day of the week (sunday, monday, etc)
    week_start: typing.Optional[str]
    # light, dark, empty is default
    theme: typing.Optional[str]
    # Selected language (beta)
    language: typing.Optional[str]
    # Explore query history preferences
    query_history: typing.Optional['QueryHistoryPreference']
    # Cookie preferences
    cookie_preferences: typing.Optional['CookiePreferences']

    def __init__(self, home_dashboard_uid: typing.Optional[str] = None, timezone: typing.Optional[str] = None, week_start: typing.Optional[str] = None, theme: typing.Optional[str] = None, language: typing.Optional[str] = None, query_history: typing.Optional['QueryHistoryPreference'] = None, cookie_preferences: typing.Optional['CookiePreferences'] = None):
        self.home_dashboard_uid = home_dashboard_uid
        self.timezone = timezone
        self.week_start = week_start
        self.theme = theme
        self.language = language
        self.query_history = query_history
        self.cookie_preferences = cookie_preferences

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.home_dashboard_uid is not None:
            payload["homeDashboardUID"] = self.home_dashboard_uid
        if self.timezone is not None:
            payload["timezone"] = self.timezone
        if self.week_start is not None:
            payload["weekStart"] = self.week_start
        if self.theme is not None:
            payload["theme"] = self.theme
        if self.language is not None:
            payload["language"] = self.language
        if self.query_history is not None:
            payload["queryHistory"] = self.query_history
        if self.cookie_preferences is not None:
            payload["cookiePreferences"] = self.cookie_preferences
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "homeDashboardUID" in data:
            args["home_dashboard_uid"] = data["homeDashboardUID"]
        if "timezone" in data:
            args["timezone"] = data["timezone"]
        if "weekStart" in data:
            args["week_start"] = data["weekStart"]
        if "theme" in data:
            args["theme"] = data["theme"]
        if "language" in data:
            args["language"] = data["language"]
        if "queryHistory" in data:
            args["query_history"] = QueryHistoryPreference.from_json(data["queryHistory"])
        if "cookiePreferences" in data:
            args["cookie_preferences"] = CookiePreferences.from_json(data["cookiePreferences"])        

        return cls(**args)


class QueryHistoryPreference:
    # one of: '' | 'query' | 'starred';
    home_tab: typing.Optional[str]

    def __init__(self, home_tab: typing.Optional[str] = None):
        self.home_tab = home_tab

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.home_tab is not None:
            payload["homeTab"] = self.home_tab
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "homeTab" in data:
            args["home_tab"] = data["homeTab"]        

        return cls(**args)


class CookiePreferences:
    analytics: typing.Optional[object]
    performance: typing.Optional[object]
    functional: typing.Optional[object]

    def __init__(self, analytics: typing.Optional[object] = None, performance: typing.Optional[object] = None, functional: typing.Optional[object] = None):
        self.analytics = analytics
        self.performance = performance
        self.functional = functional

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.analytics is not None:
            payload["analytics"] = self.analytics
        if self.performance is not None:
            payload["performance"] = self.performance
        if self.functional is not None:
            payload["functional"] = self.functional
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "analytics" in data:
            args["analytics"] = data["analytics"]
        if "performance" in data:
            args["performance"] = data["performance"]
        if "functional" in data:
            args["functional"] = data["functional"]        

        return cls(**args)






# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
from ..cog import variants as cogvariants
from ..cog import runtime as cogruntime
import enum


class Query:
    # Grafana data source unique identifier; it should be '__expr__' for a Server Side Expression operation.
    datasource_uid: typing.Optional[str]
    # JSON is the raw JSON query and includes the above properties as well as custom properties.
    model: typing.Optional[cogvariants.Dataquery]
    # QueryType is an optional identifier for the type of query.
    # It can be used to distinguish different types of queries.
    query_type: typing.Optional[str]
    # RefID is the unique identifier of the query, set by the frontend call.
    ref_id: typing.Optional[str]
    # RelativeTimeRange is the per query start and end time
    # for requests.
    relative_time_range: typing.Optional['RelativeTimeRange']

    def __init__(self, datasource_uid: typing.Optional[str] = None, model: typing.Optional[cogvariants.Dataquery] = None, query_type: typing.Optional[str] = None, ref_id: typing.Optional[str] = None, relative_time_range: typing.Optional['RelativeTimeRange'] = None):
        self.datasource_uid = datasource_uid
        self.model = model
        self.query_type = query_type
        self.ref_id = ref_id
        self.relative_time_range = relative_time_range

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.datasource_uid is not None:
            payload["datasourceUid"] = self.datasource_uid
        if self.model is not None:
            payload["model"] = self.model
        if self.query_type is not None:
            payload["queryType"] = self.query_type
        if self.ref_id is not None:
            payload["refId"] = self.ref_id
        if self.relative_time_range is not None:
            payload["relativeTimeRange"] = self.relative_time_range
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "datasourceUid" in data:
            args["datasource_uid"] = data["datasourceUid"]
        if "model" in data:
            args["model"] = cogruntime.dataquery_from_json(data["model"], "")
        if "queryType" in data:
            args["query_type"] = data["queryType"]
        if "refId" in data:
            args["ref_id"] = data["refId"]
        if "relativeTimeRange" in data:
            args["relative_time_range"] = RelativeTimeRange.from_json(data["relativeTimeRange"])        

        return cls(**args)


class RuleGroup:
    folder_uid: typing.Optional[str]
    # The interval, in seconds, at which all rules in the group are evaluated.
    # If a group contains many rules, the rules are evaluated sequentially.
    interval: typing.Optional['Duration']
    rules: typing.Optional[list['Rule']]
    title: typing.Optional[str]

    def __init__(self, folder_uid: typing.Optional[str] = None, interval: typing.Optional['Duration'] = None, rules: typing.Optional[list['Rule']] = None, title: typing.Optional[str] = None):
        self.folder_uid = folder_uid
        self.interval = interval
        self.rules = rules
        self.title = title

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.folder_uid is not None:
            payload["folderUid"] = self.folder_uid
        if self.interval is not None:
            payload["interval"] = self.interval
        if self.rules is not None:
            payload["rules"] = self.rules
        if self.title is not None:
            payload["title"] = self.title
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "folderUid" in data:
            args["folder_uid"] = data["folderUid"]
        if "interval" in data:
            args["interval"] = data["interval"]
        if "rules" in data:
            args["rules"] = [Rule.from_json(item) for item in data["rules"]]
        if "title" in data:
            args["title"] = data["title"]        

        return cls(**args)


# Duration in seconds.
Duration: typing.TypeAlias = int


class ContactPoint:
    """
    EmbeddedContactPoint is the contact point type that is used
    by grafanas embedded alertmanager implementation.
    """

    disable_resolve_message: typing.Optional[bool]
    # Name is used as grouping key in the UI. Contact points with the
    # same name will be grouped in the UI.
    name: typing.Optional[str]
    provenance: typing.Optional[str]
    settings: 'Json'
    type_val: typing.Literal["alertmanager", "dingding", "discord", "email", "googlechat", "kafka", "line", "opsgenie", "pagerduty", "pushover", "sensugo", "slack", "teams", "telegram", "threema", "victorops", "webhook", "wecom"]
    # UID is the unique identifier of the contact point. The UID can be
    # set by the user.
    uid: typing.Optional[str]

    def __init__(self, disable_resolve_message: typing.Optional[bool] = None, name: typing.Optional[str] = None, provenance: typing.Optional[str] = None, settings: typing.Optional['Json'] = None, type_val: typing.Optional[typing.Literal["alertmanager", "dingding", "discord", "email", "googlechat", "kafka", "line", "opsgenie", "pagerduty", "pushover", "sensugo", "slack", "teams", "telegram", "threema", "victorops", "webhook", "wecom"]] = None, uid: typing.Optional[str] = None):
        self.disable_resolve_message = disable_resolve_message
        self.name = name
        self.provenance = provenance
        self.settings = settings if settings is not None else Json()
        self.type_val = type_val if type_val is not None else "alertmanager"
        self.uid = uid

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "settings": self.settings,
            "type": self.type_val,
        }
        if self.disable_resolve_message is not None:
            payload["disableResolveMessage"] = self.disable_resolve_message
        if self.name is not None:
            payload["name"] = self.name
        if self.provenance is not None:
            payload["provenance"] = self.provenance
        if self.uid is not None:
            payload["uid"] = self.uid
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "disableResolveMessage" in data:
            args["disable_resolve_message"] = data["disableResolveMessage"]
        if "name" in data:
            args["name"] = data["name"]
        if "provenance" in data:
            args["provenance"] = data["provenance"]
        if "settings" in data:
            args["settings"] = data["settings"]
        if "type" in data:
            args["type_val"] = data["type"]
        if "uid" in data:
            args["uid"] = data["uid"]        

        return cls(**args)


Json: typing.TypeAlias = object


MatchRegexps: typing.TypeAlias = dict[str, str]


class MatchType(enum.StrEnum):
    EQUAL = "="
    NOT_EQUAL = "!="
    EQUAL_REGEX = "=~"
    NOT_EQUAL_REGEX = "!~"


class Matcher:
    name: typing.Optional[str]
    type: typing.Optional['MatchType']
    value: typing.Optional[str]

    def __init__(self, name: typing.Optional[str] = None, type: typing.Optional['MatchType'] = None, value: typing.Optional[str] = None):
        self.name = name
        self.type = type
        self.value = value

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.name is not None:
            payload["Name"] = self.name
        if self.type is not None:
            payload["Type"] = self.type
        if self.value is not None:
            payload["Value"] = self.value
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "Name" in data:
            args["name"] = data["Name"]
        if "Type" in data:
            args["type"] = data["Type"]
        if "Value" in data:
            args["value"] = data["Value"]        

        return cls(**args)


# Matchers is a slice of Matchers that is sortable, implements Stringer, and
# provides a Matches method to match a LabelSet against all Matchers in the
# slice. Note that some users of Matchers might require it to be sorted.
Matchers: typing.TypeAlias = list['Matcher']


class MuteTiming:
    name: typing.Optional[str]
    time_intervals: typing.Optional[list['TimeInterval']]

    def __init__(self, name: typing.Optional[str] = None, time_intervals: typing.Optional[list['TimeInterval']] = None):
        self.name = name
        self.time_intervals = time_intervals

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.name is not None:
            payload["name"] = self.name
        if self.time_intervals is not None:
            payload["time_intervals"] = self.time_intervals
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "name" in data:
            args["name"] = data["name"]
        if "time_intervals" in data:
            args["time_intervals"] = [TimeInterval.from_json(item) for item in data["time_intervals"]]        

        return cls(**args)


class NotificationTemplate:
    name: typing.Optional[str]
    provenance: typing.Optional['Provenance']
    template: typing.Optional[str]

    def __init__(self, name: typing.Optional[str] = None, provenance: typing.Optional['Provenance'] = None, template: typing.Optional[str] = None):
        self.name = name
        self.provenance = provenance
        self.template = template

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.name is not None:
            payload["name"] = self.name
        if self.provenance is not None:
            payload["provenance"] = self.provenance
        if self.template is not None:
            payload["template"] = self.template
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "name" in data:
            args["name"] = data["name"]
        if "provenance" in data:
            args["provenance"] = data["provenance"]
        if "template" in data:
            args["template"] = data["template"]        

        return cls(**args)


# Matchers is a slice of Matchers that is sortable, implements Stringer, and
# provides a Matches method to match a LabelSet against all Matchers in the
# slice. Note that some users of Matchers might require it to be sorted.
ObjectMatchers: typing.TypeAlias = 'Matchers'


Provenance: typing.TypeAlias = str


class Rule:
    annotations: typing.Optional[dict[str, str]]
    condition: str
    data: list['Query']
    exec_err_state: typing.Literal["OK", "Alerting", "Error"]
    folder_uid: str
    # The amount of time, in seconds, for which the rule must be breached for the rule to be considered to be Firing.
    # Before this time has elapsed, the rule is only considered to be Pending.
    for_val: str
    id_val: typing.Optional[int]
    is_paused: typing.Optional[bool]
    labels: typing.Optional[dict[str, str]]
    no_data_state: typing.Literal["Alerting", "NoData", "OK"]
    org_id: int
    provenance: typing.Optional['Provenance']
    rule_group: str
    title: str
    uid: typing.Optional[str]
    updated: typing.Optional[str]

    def __init__(self, annotations: typing.Optional[dict[str, str]] = None, condition: str = "", data: typing.Optional[list['Query']] = None, exec_err_state: typing.Optional[typing.Literal["OK", "Alerting", "Error"]] = None, folder_uid: str = "", for_val: str = "", id_val: typing.Optional[int] = None, is_paused: typing.Optional[bool] = None, labels: typing.Optional[dict[str, str]] = None, no_data_state: typing.Optional[typing.Literal["Alerting", "NoData", "OK"]] = None, org_id: int = 0, provenance: typing.Optional['Provenance'] = None, rule_group: str = "", title: str = "", uid: typing.Optional[str] = None, updated: typing.Optional[str] = None):
        self.annotations = annotations
        self.condition = condition
        self.data = data if data is not None else []
        self.exec_err_state = exec_err_state if exec_err_state is not None else "OK"
        self.folder_uid = folder_uid
        self.for_val = for_val
        self.id_val = id_val
        self.is_paused = is_paused
        self.labels = labels
        self.no_data_state = no_data_state if no_data_state is not None else "Alerting"
        self.org_id = org_id
        self.provenance = provenance
        self.rule_group = rule_group
        self.title = title
        self.uid = uid
        self.updated = updated

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "condition": self.condition,
            "data": self.data,
            "execErrState": self.exec_err_state,
            "folderUID": self.folder_uid,
            "for": self.for_val,
            "noDataState": self.no_data_state,
            "orgID": self.org_id,
            "ruleGroup": self.rule_group,
            "title": self.title,
        }
        if self.annotations is not None:
            payload["annotations"] = self.annotations
        if self.id_val is not None:
            payload["id"] = self.id_val
        if self.is_paused is not None:
            payload["isPaused"] = self.is_paused
        if self.labels is not None:
            payload["labels"] = self.labels
        if self.provenance is not None:
            payload["provenance"] = self.provenance
        if self.uid is not None:
            payload["uid"] = self.uid
        if self.updated is not None:
            payload["updated"] = self.updated
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "annotations" in data:
            args["annotations"] = data["annotations"]
        if "condition" in data:
            args["condition"] = data["condition"]
        if "data" in data:
            args["data"] = [Query.from_json(item) for item in data["data"]]
        if "execErrState" in data:
            args["exec_err_state"] = data["execErrState"]
        if "folderUID" in data:
            args["folder_uid"] = data["folderUID"]
        if "for" in data:
            args["for_val"] = data["for"]
        if "id" in data:
            args["id_val"] = data["id"]
        if "isPaused" in data:
            args["is_paused"] = data["isPaused"]
        if "labels" in data:
            args["labels"] = data["labels"]
        if "noDataState" in data:
            args["no_data_state"] = data["noDataState"]
        if "orgID" in data:
            args["org_id"] = data["orgID"]
        if "provenance" in data:
            args["provenance"] = data["provenance"]
        if "ruleGroup" in data:
            args["rule_group"] = data["ruleGroup"]
        if "title" in data:
            args["title"] = data["title"]
        if "uid" in data:
            args["uid"] = data["uid"]
        if "updated" in data:
            args["updated"] = data["updated"]        

        return cls(**args)


class RelativeTimeRange:
    """
    RelativeTimeRange is the per query start and end time
    for requests.
    """

    # A Duration represents the elapsed time between two instants
    # as an int64 nanosecond count. The representation limits the
    # largest representable duration to approximately 290 years.
    from_val: typing.Optional['Duration']
    # A Duration represents the elapsed time between two instants
    # as an int64 nanosecond count. The representation limits the
    # largest representable duration to approximately 290 years.
    to: typing.Optional['Duration']

    def __init__(self, from_val: typing.Optional['Duration'] = None, to: typing.Optional['Duration'] = None):
        self.from_val = from_val
        self.to = to

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.from_val is not None:
            payload["from"] = self.from_val
        if self.to is not None:
            payload["to"] = self.to
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "from" in data:
            args["from_val"] = data["from"]
        if "to" in data:
            args["to"] = data["to"]        

        return cls(**args)


class NotificationPolicy:
    """
    A Route is a node that contains definitions of how to handle alerts. This is modified
    from the upstream alertmanager in that it adds the ObjectMatchers property.
    """

    continue_val: typing.Optional[bool]
    group_by: typing.Optional[list[str]]
    group_interval: typing.Optional[str]
    group_wait: typing.Optional[str]
    # Deprecated. Remove before v1.0 release.
    match: typing.Optional[dict[str, str]]
    match_re: typing.Optional['MatchRegexps']
    # Matchers is a slice of Matchers that is sortable, implements Stringer, and
    # provides a Matches method to match a LabelSet against all Matchers in the
    # slice. Note that some users of Matchers might require it to be sorted.
    matchers: typing.Optional['Matchers']
    mute_time_intervals: typing.Optional[list[str]]
    # Matchers is a slice of Matchers that is sortable, implements Stringer, and
    # provides a Matches method to match a LabelSet against all Matchers in the
    # slice. Note that some users of Matchers might require it to be sorted.
    object_matchers: typing.Optional['ObjectMatchers']
    provenance: typing.Optional['Provenance']
    receiver: typing.Optional[str]
    repeat_interval: typing.Optional[str]
    routes: typing.Optional[list['NotificationPolicy']]

    def __init__(self, continue_val: typing.Optional[bool] = None, group_by: typing.Optional[list[str]] = None, group_interval: typing.Optional[str] = None, group_wait: typing.Optional[str] = None, match: typing.Optional[dict[str, str]] = None, match_re: typing.Optional['MatchRegexps'] = None, matchers: typing.Optional['Matchers'] = None, mute_time_intervals: typing.Optional[list[str]] = None, object_matchers: typing.Optional['ObjectMatchers'] = None, provenance: typing.Optional['Provenance'] = None, receiver: typing.Optional[str] = None, repeat_interval: typing.Optional[str] = None, routes: typing.Optional[list['NotificationPolicy']] = None):
        self.continue_val = continue_val
        self.group_by = group_by
        self.group_interval = group_interval
        self.group_wait = group_wait
        self.match = match
        self.match_re = match_re
        self.matchers = matchers
        self.mute_time_intervals = mute_time_intervals
        self.object_matchers = object_matchers
        self.provenance = provenance
        self.receiver = receiver
        self.repeat_interval = repeat_interval
        self.routes = routes

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.continue_val is not None:
            payload["continue"] = self.continue_val
        if self.group_by is not None:
            payload["group_by"] = self.group_by
        if self.group_interval is not None:
            payload["group_interval"] = self.group_interval
        if self.group_wait is not None:
            payload["group_wait"] = self.group_wait
        if self.match is not None:
            payload["match"] = self.match
        if self.match_re is not None:
            payload["match_re"] = self.match_re
        if self.matchers is not None:
            payload["matchers"] = self.matchers
        if self.mute_time_intervals is not None:
            payload["mute_time_intervals"] = self.mute_time_intervals
        if self.object_matchers is not None:
            payload["object_matchers"] = self.object_matchers
        if self.provenance is not None:
            payload["provenance"] = self.provenance
        if self.receiver is not None:
            payload["receiver"] = self.receiver
        if self.repeat_interval is not None:
            payload["repeat_interval"] = self.repeat_interval
        if self.routes is not None:
            payload["routes"] = self.routes
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "continue" in data:
            args["continue_val"] = data["continue"]
        if "group_by" in data:
            args["group_by"] = data["group_by"]
        if "group_interval" in data:
            args["group_interval"] = data["group_interval"]
        if "group_wait" in data:
            args["group_wait"] = data["group_wait"]
        if "match" in data:
            args["match"] = data["match"]
        if "match_re" in data:
            args["match_re"] = data["match_re"]
        if "matchers" in data:
            args["matchers"] = [Matcher.from_json(item) for item in data["matchers"]]
        if "mute_time_intervals" in data:
            args["mute_time_intervals"] = data["mute_time_intervals"]
        if "object_matchers" in data:
            args["object_matchers"] = [Matcher.from_json(item) for item in data["object_matchers"]]
        if "provenance" in data:
            args["provenance"] = data["provenance"]
        if "receiver" in data:
            args["receiver"] = data["receiver"]
        if "repeat_interval" in data:
            args["repeat_interval"] = data["repeat_interval"]
        if "routes" in data:
            args["routes"] = [NotificationPolicy.from_json(item) for item in data["routes"]]        

        return cls(**args)


class TimeInterval:
    times: typing.Optional[list['TimeRange']]
    weekdays: typing.Optional[list['WeekdayRange']]
    days_of_month: typing.Optional[list['DayOfMonthRange']]
    months: typing.Optional[list['MonthRange']]
    years: typing.Optional[list['YearRange']]
    location: typing.Optional['Location']

    def __init__(self, times: typing.Optional[list['TimeRange']] = None, weekdays: typing.Optional[list['WeekdayRange']] = None, days_of_month: typing.Optional[list['DayOfMonthRange']] = None, months: typing.Optional[list['MonthRange']] = None, years: typing.Optional[list['YearRange']] = None, location: typing.Optional['Location'] = None):
        self.times = times
        self.weekdays = weekdays
        self.days_of_month = days_of_month
        self.months = months
        self.years = years
        self.location = location

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.times is not None:
            payload["times"] = self.times
        if self.weekdays is not None:
            payload["weekdays"] = self.weekdays
        if self.days_of_month is not None:
            payload["days_of_month"] = self.days_of_month
        if self.months is not None:
            payload["months"] = self.months
        if self.years is not None:
            payload["years"] = self.years
        if self.location is not None:
            payload["location"] = self.location
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "times" in data:
            args["times"] = [TimeRange.from_json(item) for item in data["times"]]
        if "weekdays" in data:
            args["weekdays"] = [WeekdayRange.from_json(item) for item in data["weekdays"]]
        if "days_of_month" in data:
            args["days_of_month"] = [DayOfMonthRange.from_json(item) for item in data["days_of_month"]]
        if "months" in data:
            args["months"] = [MonthRange.from_json(item) for item in data["months"]]
        if "years" in data:
            args["years"] = [YearRange.from_json(item) for item in data["years"]]
        if "location" in data:
            args["location"] = data["location"]        

        return cls(**args)


class TimeRange:
    """
    Redefining this to avoid an import cycle
    """

    from_val: typing.Optional[str]
    to: typing.Optional[str]

    def __init__(self, from_val: typing.Optional[str] = None, to: typing.Optional[str] = None):
        self.from_val = from_val
        self.to = to

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.from_val is not None:
            payload["from"] = self.from_val
        if self.to is not None:
            payload["to"] = self.to
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "from" in data:
            args["from_val"] = data["from"]
        if "to" in data:
            args["to"] = data["to"]        

        return cls(**args)


class WeekdayRange:
    begin: typing.Optional[int]
    end: typing.Optional[int]

    def __init__(self, begin: typing.Optional[int] = None, end: typing.Optional[int] = None):
        self.begin = begin
        self.end = end

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.begin is not None:
            payload["begin"] = self.begin
        if self.end is not None:
            payload["end"] = self.end
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "begin" in data:
            args["begin"] = data["begin"]
        if "end" in data:
            args["end"] = data["end"]        

        return cls(**args)


class DayOfMonthRange:
    begin: typing.Optional[int]
    end: typing.Optional[int]

    def __init__(self, begin: typing.Optional[int] = None, end: typing.Optional[int] = None):
        self.begin = begin
        self.end = end

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.begin is not None:
            payload["begin"] = self.begin
        if self.end is not None:
            payload["end"] = self.end
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "begin" in data:
            args["begin"] = data["begin"]
        if "end" in data:
            args["end"] = data["end"]        

        return cls(**args)


class YearRange:
    begin: typing.Optional[int]
    end: typing.Optional[int]

    def __init__(self, begin: typing.Optional[int] = None, end: typing.Optional[int] = None):
        self.begin = begin
        self.end = end

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.begin is not None:
            payload["begin"] = self.begin
        if self.end is not None:
            payload["end"] = self.end
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "begin" in data:
            args["begin"] = data["begin"]
        if "end" in data:
            args["end"] = data["end"]        

        return cls(**args)


class MonthRange:
    begin: typing.Optional[int]
    end: typing.Optional[int]

    def __init__(self, begin: typing.Optional[int] = None, end: typing.Optional[int] = None):
        self.begin = begin
        self.end = end

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.begin is not None:
            payload["begin"] = self.begin
        if self.end is not None:
            payload["end"] = self.end
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "begin" in data:
            args["begin"] = data["begin"]
        if "end" in data:
            args["end"] = data["end"]        

        return cls(**args)


Location: typing.TypeAlias = str






# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import enum
from ..models import common
import typing
from ..cog import runtime as cogruntime


class PieChartType(enum.StrEnum):
    """
    Select the pie chart display style.
    """

    PIE = "pie"
    DONUT = "donut"


class PieChartLabels(enum.StrEnum):
    """
    Select labels to display on the pie chart.
     - Name - The series or field name.
     - Percent - The percentage of the whole.
     - Value - The raw numerical value.
    """

    NAME = "name"
    VALUE = "value"
    PERCENT = "percent"


class PieChartLegendValues(enum.StrEnum):
    """
    Select values to display in the legend.
     - Percent: The percentage of the whole.
     - Value: The raw numerical value.
    """

    VALUE = "value"
    PERCENT = "percent"


class PieChartLegendOptions:
    values: list['PieChartLegendValues']
    display_mode: common.LegendDisplayMode
    placement: common.LegendPlacement
    show_legend: bool
    as_table: typing.Optional[bool]
    is_visible: typing.Optional[bool]
    sort_by: typing.Optional[str]
    sort_desc: typing.Optional[bool]
    width: typing.Optional[float]
    calcs: list[str]

    def __init__(self, values: typing.Optional[list['PieChartLegendValues']] = None, display_mode: typing.Optional[common.LegendDisplayMode] = None, placement: typing.Optional[common.LegendPlacement] = None, show_legend: bool = False, as_table: typing.Optional[bool] = None, is_visible: typing.Optional[bool] = None, sort_by: typing.Optional[str] = None, sort_desc: typing.Optional[bool] = None, width: typing.Optional[float] = None, calcs: typing.Optional[list[str]] = None):
        self.values = values if values is not None else []
        self.display_mode = display_mode if display_mode is not None else common.LegendDisplayMode.LIST
        self.placement = placement if placement is not None else common.LegendPlacement.BOTTOM
        self.show_legend = show_legend
        self.as_table = as_table
        self.is_visible = is_visible
        self.sort_by = sort_by
        self.sort_desc = sort_desc
        self.width = width
        self.calcs = calcs if calcs is not None else []

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "values": self.values,
            "displayMode": self.display_mode,
            "placement": self.placement,
            "showLegend": self.show_legend,
            "calcs": self.calcs,
        }
        if self.as_table is not None:
            payload["asTable"] = self.as_table
        if self.is_visible is not None:
            payload["isVisible"] = self.is_visible
        if self.sort_by is not None:
            payload["sortBy"] = self.sort_by
        if self.sort_desc is not None:
            payload["sortDesc"] = self.sort_desc
        if self.width is not None:
            payload["width"] = self.width
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "values" in data:
            args["values"] = [item for item in data["values"]]
        if "displayMode" in data:
            args["display_mode"] = data["displayMode"]
        if "placement" in data:
            args["placement"] = data["placement"]
        if "showLegend" in data:
            args["show_legend"] = data["showLegend"]
        if "asTable" in data:
            args["as_table"] = data["asTable"]
        if "isVisible" in data:
            args["is_visible"] = data["isVisible"]
        if "sortBy" in data:
            args["sort_by"] = data["sortBy"]
        if "sortDesc" in data:
            args["sort_desc"] = data["sortDesc"]
        if "width" in data:
            args["width"] = data["width"]
        if "calcs" in data:
            args["calcs"] = data["calcs"]        

        return cls(**args)


class Options:
    pie_type: 'PieChartType'
    display_labels: typing.Optional[list['PieChartLabels']]
    tooltip: common.VizTooltipOptions
    reduce_options: common.ReduceDataOptions
    text: typing.Optional[common.VizTextDisplayOptions]
    legend: 'PieChartLegendOptions'
    orientation: common.VizOrientation

    def __init__(self, pie_type: typing.Optional['PieChartType'] = None, display_labels: typing.Optional[list['PieChartLabels']] = None, tooltip: typing.Optional[common.VizTooltipOptions] = None, reduce_options: typing.Optional[common.ReduceDataOptions] = None, text: typing.Optional[common.VizTextDisplayOptions] = None, legend: typing.Optional['PieChartLegendOptions'] = None, orientation: typing.Optional[common.VizOrientation] = None):
        self.pie_type = pie_type if pie_type is not None else PieChartType.PIE
        self.display_labels = display_labels
        self.tooltip = tooltip if tooltip is not None else common.VizTooltipOptions()
        self.reduce_options = reduce_options if reduce_options is not None else common.ReduceDataOptions()
        self.text = text
        self.legend = legend if legend is not None else PieChartLegendOptions()
        self.orientation = orientation if orientation is not None else common.VizOrientation.AUTO

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "pieType": self.pie_type,
            "tooltip": self.tooltip,
            "reduceOptions": self.reduce_options,
            "legend": self.legend,
            "orientation": self.orientation,
        }
        if self.display_labels is not None:
            payload["displayLabels"] = self.display_labels
        if self.text is not None:
            payload["text"] = self.text
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "pieType" in data:
            args["pie_type"] = data["pieType"]
        if "displayLabels" in data:
            args["display_labels"] = [item for item in data["displayLabels"]]
        if "tooltip" in data:
            args["tooltip"] = common.VizTooltipOptions.from_json(data["tooltip"])
        if "reduceOptions" in data:
            args["reduce_options"] = common.ReduceDataOptions.from_json(data["reduceOptions"])
        if "text" in data:
            args["text"] = common.VizTextDisplayOptions.from_json(data["text"])
        if "legend" in data:
            args["legend"] = PieChartLegendOptions.from_json(data["legend"])
        if "orientation" in data:
            args["orientation"] = data["orientation"]        

        return cls(**args)


FieldConfig: typing.TypeAlias = common.HideableFieldConfig





def variant_config() -> cogruntime.PanelCfgConfig:
    return cogruntime.PanelCfgConfig(
        identifier="piechart",
        options_from_json_hook=Options.from_json,
        field_config_from_json_hook=FieldConfig.from_json,
    )




# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import enum
import typing
from ..cog import variants as cogvariants
from ..cog import runtime as cogruntime


class TestDataQueryType(enum.StrEnum):
    RANDOM_WALK = "random_walk"
    SLOW_QUERY = "slow_query"
    RANDOM_WALK_WITH_ERROR = "random_walk_with_error"
    RANDOM_WALK_TABLE = "random_walk_table"
    EXPONENTIAL_HEATMAP_BUCKET_DATA = "exponential_heatmap_bucket_data"
    LINEAR_HEATMAP_BUCKET_DATA = "linear_heatmap_bucket_data"
    NO_DATA_POINTS = "no_data_points"
    DATA_POINTS_OUTSIDE_RANGE = "datapoints_outside_range"
    CSV_METRIC_VALUES = "csv_metric_values"
    PREDICTABLE_PULSE = "predictable_pulse"
    PREDICTABLE_CSV_WAVE = "predictable_csv_wave"
    STREAMING_CLIENT = "streaming_client"
    SIMULATION = "simulation"
    USA = "usa"
    LIVE = "live"
    GRAFANA_API = "grafana_api"
    ARROW = "arrow"
    ANNOTATIONS = "annotations"
    TABLE_STATIC = "table_static"
    SERVER_ERROR500 = "server_error_500"
    LOGS = "logs"
    NODE_GRAPH = "node_graph"
    FLAME_GRAPH = "flame_graph"
    RAW_FRAME = "raw_frame"
    CSV_FILE = "csv_file"
    CSV_CONTENT = "csv_content"
    TRACE = "trace"
    MANUAL_ENTRY = "manual_entry"
    VARIABLES_QUERY = "variables-query"


class StreamingQuery:
    type_val: typing.Literal["signal", "logs", "fetch"]
    speed: int
    spread: int
    noise: int
    bands: typing.Optional[int]
    url: typing.Optional[str]

    def __init__(self, type_val: typing.Optional[typing.Literal["signal", "logs", "fetch"]] = None, speed: int = 0, spread: int = 0, noise: int = 0, bands: typing.Optional[int] = None, url: typing.Optional[str] = None):
        self.type_val = type_val if type_val is not None else "signal"
        self.speed = speed
        self.spread = spread
        self.noise = noise
        self.bands = bands
        self.url = url

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
            "speed": self.speed,
            "spread": self.spread,
            "noise": self.noise,
        }
        if self.bands is not None:
            payload["bands"] = self.bands
        if self.url is not None:
            payload["url"] = self.url
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "type" in data:
            args["type_val"] = data["type"]
        if "speed" in data:
            args["speed"] = data["speed"]
        if "spread" in data:
            args["spread"] = data["spread"]
        if "noise" in data:
            args["noise"] = data["noise"]
        if "bands" in data:
            args["bands"] = data["bands"]
        if "url" in data:
            args["url"] = data["url"]        

        return cls(**args)


class PulseWaveQuery:
    time_step: typing.Optional[int]
    on_count: typing.Optional[int]
    off_count: typing.Optional[int]
    on_value: typing.Optional[float]
    off_value: typing.Optional[float]

    def __init__(self, time_step: typing.Optional[int] = None, on_count: typing.Optional[int] = None, off_count: typing.Optional[int] = None, on_value: typing.Optional[float] = None, off_value: typing.Optional[float] = None):
        self.time_step = time_step
        self.on_count = on_count
        self.off_count = off_count
        self.on_value = on_value
        self.off_value = off_value

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.time_step is not None:
            payload["timeStep"] = self.time_step
        if self.on_count is not None:
            payload["onCount"] = self.on_count
        if self.off_count is not None:
            payload["offCount"] = self.off_count
        if self.on_value is not None:
            payload["onValue"] = self.on_value
        if self.off_value is not None:
            payload["offValue"] = self.off_value
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "timeStep" in data:
            args["time_step"] = data["timeStep"]
        if "onCount" in data:
            args["on_count"] = data["onCount"]
        if "offCount" in data:
            args["off_count"] = data["offCount"]
        if "onValue" in data:
            args["on_value"] = data["onValue"]
        if "offValue" in data:
            args["off_value"] = data["offValue"]        

        return cls(**args)


class SimulationQuery:
    key: 'Key'
    config: typing.Optional[dict[str, object]]
    stream: typing.Optional[bool]
    last: typing.Optional[bool]

    def __init__(self, key: typing.Optional['Key'] = None, config: typing.Optional[dict[str, object]] = None, stream: typing.Optional[bool] = None, last: typing.Optional[bool] = None):
        self.key = key if key is not None else Key()
        self.config = config
        self.stream = stream
        self.last = last

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "key": self.key,
        }
        if self.config is not None:
            payload["config"] = self.config
        if self.stream is not None:
            payload["stream"] = self.stream
        if self.last is not None:
            payload["last"] = self.last
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "key" in data:
            args["key"] = Key.from_json(data["key"])
        if "config" in data:
            args["config"] = data["config"]
        if "stream" in data:
            args["stream"] = data["stream"]
        if "last" in data:
            args["last"] = data["last"]        

        return cls(**args)


class CSVWave:
    time_step: typing.Optional[int]
    name: typing.Optional[str]
    values_csv: typing.Optional[str]
    labels: typing.Optional[str]

    def __init__(self, time_step: typing.Optional[int] = None, name: typing.Optional[str] = None, values_csv: typing.Optional[str] = None, labels: typing.Optional[str] = None):
        self.time_step = time_step
        self.name = name
        self.values_csv = values_csv
        self.labels = labels

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.time_step is not None:
            payload["timeStep"] = self.time_step
        if self.name is not None:
            payload["name"] = self.name
        if self.values_csv is not None:
            payload["valuesCSV"] = self.values_csv
        if self.labels is not None:
            payload["labels"] = self.labels
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "timeStep" in data:
            args["time_step"] = data["timeStep"]
        if "name" in data:
            args["name"] = data["name"]
        if "valuesCSV" in data:
            args["values_csv"] = data["valuesCSV"]
        if "labels" in data:
            args["labels"] = data["labels"]        

        return cls(**args)


class NodesQuery:
    type_val: typing.Optional[typing.Literal["random", "response", "random edges"]]
    count: typing.Optional[int]

    def __init__(self, type_val: typing.Optional[typing.Literal["random", "response", "random edges"]] = None, count: typing.Optional[int] = None):
        self.type_val = type_val
        self.count = count

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.type_val is not None:
            payload["type"] = self.type_val
        if self.count is not None:
            payload["count"] = self.count
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "type" in data:
            args["type_val"] = data["type"]
        if "count" in data:
            args["count"] = data["count"]        

        return cls(**args)


class USAQuery:
    mode: typing.Optional[str]
    period: typing.Optional[str]
    fields: typing.Optional[list[str]]
    states: typing.Optional[list[str]]

    def __init__(self, mode: typing.Optional[str] = None, period: typing.Optional[str] = None, fields: typing.Optional[list[str]] = None, states: typing.Optional[list[str]] = None):
        self.mode = mode
        self.period = period
        self.fields = fields
        self.states = states

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.mode is not None:
            payload["mode"] = self.mode
        if self.period is not None:
            payload["period"] = self.period
        if self.fields is not None:
            payload["fields"] = self.fields
        if self.states is not None:
            payload["states"] = self.states
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "mode" in data:
            args["mode"] = data["mode"]
        if "period" in data:
            args["period"] = data["period"]
        if "fields" in data:
            args["fields"] = data["fields"]
        if "states" in data:
            args["states"] = data["states"]        

        return cls(**args)


class Scenario:
    """
    TODO: Should this live here given it's not used in the dataquery?
    """

    id_val: str
    name: str
    string_input: str
    description: typing.Optional[str]
    hide_alias_field: typing.Optional[bool]

    def __init__(self, id_val: str = "", name: str = "", string_input: str = "", description: typing.Optional[str] = None, hide_alias_field: typing.Optional[bool] = None):
        self.id_val = id_val
        self.name = name
        self.string_input = string_input
        self.description = description
        self.hide_alias_field = hide_alias_field

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "id": self.id_val,
            "name": self.name,
            "stringInput": self.string_input,
        }
        if self.description is not None:
            payload["description"] = self.description
        if self.hide_alias_field is not None:
            payload["hideAliasField"] = self.hide_alias_field
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "id" in data:
            args["id_val"] = data["id"]
        if "name" in data:
            args["name"] = data["name"]
        if "stringInput" in data:
            args["string_input"] = data["stringInput"]
        if "description" in data:
            args["description"] = data["description"]
        if "hideAliasField" in data:
            args["hide_alias_field"] = data["hideAliasField"]        

        return cls(**args)


class Dataquery(cogvariants.Dataquery):
    alias: typing.Optional[str]
    scenario_id: typing.Optional['TestDataQueryType']
    string_input: typing.Optional[str]
    stream: typing.Optional['StreamingQuery']
    pulse_wave: typing.Optional['PulseWaveQuery']
    sim: typing.Optional['SimulationQuery']
    csv_wave: typing.Optional[list['CSVWave']]
    labels: typing.Optional[str]
    lines: typing.Optional[int]
    level_column: typing.Optional[bool]
    channel: typing.Optional[str]
    nodes: typing.Optional['NodesQuery']
    csv_file_name: typing.Optional[str]
    csv_content: typing.Optional[str]
    raw_frame_content: typing.Optional[str]
    series_count: typing.Optional[int]
    usa: typing.Optional['USAQuery']
    error_type: typing.Optional[typing.Literal["server_panic", "frontend_exception", "frontend_observable"]]
    span_count: typing.Optional[int]
    points: typing.Optional[list[list[typing.Union[str, int]]]]
    # Drop percentage (the chance we will lose a point 0-100)
    drop_percent: typing.Optional[float]
    flamegraph_diff: typing.Optional[bool]
    # A unique identifier for the query within the list of targets.
    # In server side expressions, the refId is used as a variable name to identify results.
    # By default, the UI will assign A->Z; however setting meaningful names may be useful.
    ref_id: str
    # true if query is disabled (ie should not be returned to the dashboard)
    # Note this does not always imply that the query should not be executed since
    # the results from a hidden query may be used as the input to other queries (SSE etc)
    hide: typing.Optional[bool]
    # Specify the query flavor
    # TODO make this required and give it a default
    query_type: typing.Optional[str]
    # For mixed data sources the selected datasource is on the query level.
    # For non mixed scenarios this is undefined.
    # TODO find a better way to do this ^ that's friendly to schema
    # TODO this shouldn't be unknown but DataSourceRef | null
    datasource: typing.Optional[object]

    def __init__(self, alias: typing.Optional[str] = None, scenario_id: typing.Optional['TestDataQueryType'] = None, string_input: typing.Optional[str] = None, stream: typing.Optional['StreamingQuery'] = None, pulse_wave: typing.Optional['PulseWaveQuery'] = None, sim: typing.Optional['SimulationQuery'] = None, csv_wave: typing.Optional[list['CSVWave']] = None, labels: typing.Optional[str] = None, lines: typing.Optional[int] = None, level_column: typing.Optional[bool] = None, channel: typing.Optional[str] = None, nodes: typing.Optional['NodesQuery'] = None, csv_file_name: typing.Optional[str] = None, csv_content: typing.Optional[str] = None, raw_frame_content: typing.Optional[str] = None, series_count: typing.Optional[int] = None, usa: typing.Optional['USAQuery'] = None, error_type: typing.Optional[typing.Literal["server_panic", "frontend_exception", "frontend_observable"]] = None, span_count: typing.Optional[int] = None, points: typing.Optional[list[list[typing.Union[str, int]]]] = None, drop_percent: typing.Optional[float] = None, flamegraph_diff: typing.Optional[bool] = None, ref_id: str = "", hide: typing.Optional[bool] = None, query_type: typing.Optional[str] = None, datasource: typing.Optional[object] = None):
        self.alias = alias
        self.scenario_id = scenario_id if scenario_id is not None else TestDataQueryType.RANDOM_WALK
        self.string_input = string_input
        self.stream = stream
        self.pulse_wave = pulse_wave
        self.sim = sim
        self.csv_wave = csv_wave
        self.labels = labels
        self.lines = lines
        self.level_column = level_column
        self.channel = channel
        self.nodes = nodes
        self.csv_file_name = csv_file_name
        self.csv_content = csv_content
        self.raw_frame_content = raw_frame_content
        self.series_count = series_count
        self.usa = usa
        self.error_type = error_type
        self.span_count = span_count
        self.points = points
        self.drop_percent = drop_percent
        self.flamegraph_diff = flamegraph_diff
        self.ref_id = ref_id
        self.hide = hide
        self.query_type = query_type
        self.datasource = datasource

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "refId": self.ref_id,
        }
        if self.alias is not None:
            payload["alias"] = self.alias
        if self.scenario_id is not None:
            payload["scenarioId"] = self.scenario_id
        if self.string_input is not None:
            payload["stringInput"] = self.string_input
        if self.stream is not None:
            payload["stream"] = self.stream
        if self.pulse_wave is not None:
            payload["pulseWave"] = self.pulse_wave
        if self.sim is not None:
            payload["sim"] = self.sim
        if self.csv_wave is not None:
            payload["csvWave"] = self.csv_wave
        if self.labels is not None:
            payload["labels"] = self.labels
        if self.lines is not None:
            payload["lines"] = self.lines
        if self.level_column is not None:
            payload["levelColumn"] = self.level_column
        if self.channel is not None:
            payload["channel"] = self.channel
        if self.nodes is not None:
            payload["nodes"] = self.nodes
        if self.csv_file_name is not None:
            payload["csvFileName"] = self.csv_file_name
        if self.csv_content is not None:
            payload["csvContent"] = self.csv_content
        if self.raw_frame_content is not None:
            payload["rawFrameContent"] = self.raw_frame_content
        if self.series_count is not None:
            payload["seriesCount"] = self.series_count
        if self.usa is not None:
            payload["usa"] = self.usa
        if self.error_type is not None:
            payload["errorType"] = self.error_type
        if self.span_count is not None:
            payload["spanCount"] = self.span_count
        if self.points is not None:
            payload["points"] = self.points
        if self.drop_percent is not None:
            payload["dropPercent"] = self.drop_percent
        if self.flamegraph_diff is not None:
            payload["flamegraphDiff"] = self.flamegraph_diff
        if self.hide is not None:
            payload["hide"] = self.hide
        if self.query_type is not None:
            payload["queryType"] = self.query_type
        if self.datasource is not None:
            payload["datasource"] = self.datasource
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "alias" in data:
            args["alias"] = data["alias"]
        if "scenarioId" in data:
            args["scenario_id"] = data["scenarioId"]
        if "stringInput" in data:
            args["string_input"] = data["stringInput"]
        if "stream" in data:
            args["stream"] = StreamingQuery.from_json(data["stream"])
        if "pulseWave" in data:
            args["pulse_wave"] = PulseWaveQuery.from_json(data["pulseWave"])
        if "sim" in data:
            args["sim"] = SimulationQuery.from_json(data["sim"])
        if "csvWave" in data:
            args["csv_wave"] = [CSVWave.from_json(item) for item in data["csvWave"]]
        if "labels" in data:
            args["labels"] = data["labels"]
        if "lines" in data:
            args["lines"] = data["lines"]
        if "levelColumn" in data:
            args["level_column"] = data["levelColumn"]
        if "channel" in data:
            args["channel"] = data["channel"]
        if "nodes" in data:
            args["nodes"] = NodesQuery.from_json(data["nodes"])
        if "csvFileName" in data:
            args["csv_file_name"] = data["csvFileName"]
        if "csvContent" in data:
            args["csv_content"] = data["csvContent"]
        if "rawFrameContent" in data:
            args["raw_frame_content"] = data["rawFrameContent"]
        if "seriesCount" in data:
            args["series_count"] = data["seriesCount"]
        if "usa" in data:
            args["usa"] = USAQuery.from_json(data["usa"])
        if "errorType" in data:
            args["error_type"] = data["errorType"]
        if "spanCount" in data:
            args["span_count"] = data["spanCount"]
        if "points" in data:
            args["points"] = [[item for item in item] for item in data["points"]]
        if "dropPercent" in data:
            args["drop_percent"] = data["dropPercent"]
        if "flamegraphDiff" in data:
            args["flamegraph_diff"] = data["flamegraphDiff"]
        if "refId" in data:
            args["ref_id"] = data["refId"]
        if "hide" in data:
            args["hide"] = data["hide"]
        if "queryType" in data:
            args["query_type"] = data["queryType"]
        if "datasource" in data:
            args["datasource"] = data["datasource"]        

        return cls(**args)


class Key:
    type_val: str
    tick: float
    uid: typing.Optional[str]

    def __init__(self, type_val: str = "", tick: float = 0, uid: typing.Optional[str] = None):
        self.type_val = type_val
        self.tick = tick
        self.uid = uid

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
            "tick": self.tick,
        }
        if self.uid is not None:
            payload["uid"] = self.uid
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "type" in data:
            args["type_val"] = data["type"]
        if "tick" in data:
            args["tick"] = data["tick"]
        if "uid" in data:
            args["uid"] = data["uid"]        

        return cls(**args)





def variant_config() -> cogruntime.DataqueryConfig:
    return cogruntime.DataqueryConfig(
        identifier="testdata",
        from_json_hook=Dataquery.from_json,
    )




# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
from ..cog import builder as cogbuilder
from ..models import publicdashboard


class PublicDashboard(cogbuilder.Builder[publicdashboard.PublicDashboard]):
    _internal: publicdashboard.PublicDashboard

    def __init__(self):
        self._internal = publicdashboard.PublicDashboard()

    def build(self) -> publicdashboard.PublicDashboard:
        """
        Builds the object.
        """
        return self._internal    
    
    def uid(self, uid: str) -> typing.Self:    
        """
        Unique public dashboard identifier
        """
            
        self._internal.uid = uid
    
        return self
    
    def dashboard_uid(self, dashboard_uid: str) -> typing.Self:    
        """
        Dashboard unique identifier referenced by this public dashboard
        """
            
        self._internal.dashboard_uid = dashboard_uid
    
        return self
    
    def access_token(self, access_token: str) -> typing.Self:    
        """
        Unique public access token
        """
            
        self._internal.access_token = access_token
    
        return self
    
    def is_enabled(self, is_enabled: bool) -> typing.Self:    
        """
        Flag that indicates if the public dashboard is enabled
        """
            
        self._internal.is_enabled = is_enabled
    
        return self
    
    def annotations_enabled(self, annotations_enabled: bool) -> typing.Self:    
        """
        Flag that indicates if annotations are enabled
        """
            
        self._internal.annotations_enabled = annotations_enabled
    
        return self
    
    def time_selection_enabled(self, time_selection_enabled: bool) -> typing.Self:    
        """
        Flag that indicates if the time range picker is enabled
        """
            
        self._internal.time_selection_enabled = time_selection_enabled
    
        return self
    



# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
from ..cog import builder as cogbuilder
from ..models import tempo
from ..models import dashboard


class TempoQuery(cogbuilder.Builder[tempo.TempoQuery]):
    _internal: tempo.TempoQuery

    def __init__(self):
        self._internal = tempo.TempoQuery()

    def build(self) -> tempo.TempoQuery:
        """
        Builds the object.
        """
        return self._internal    
    
    def ref_id(self, ref_id: str) -> typing.Self:    
        """
        A unique identifier for the query within the list of targets.
        In server side expressions, the refId is used as a variable name to identify results.
        By default, the UI will assign A->Z; however setting meaningful names may be useful.
        """
            
        self._internal.ref_id = ref_id
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        """
        true if query is disabled (ie should not be returned to the dashboard)
        Note this does not always imply that the query should not be executed since
        the results from a hidden query may be used as the input to other queries (SSE etc)
        """
            
        self._internal.hide = hide
    
        return self
    
    def query_type(self, query_type: str) -> typing.Self:    
        """
        Specify the query flavor
        TODO make this required and give it a default
        """
            
        self._internal.query_type = query_type
    
        return self
    
    def query(self, query: str) -> typing.Self:    
        """
        TraceQL query or trace ID
        """
            
        self._internal.query = query
    
        return self
    
    def search(self, search: str) -> typing.Self:    
        """
        @deprecated Logfmt query to filter traces by their tags. Example: http.status_code=200 error=true
        """
            
        self._internal.search = search
    
        return self
    
    def service_name(self, service_name: str) -> typing.Self:    
        """
        @deprecated Query traces by service name
        """
            
        self._internal.service_name = service_name
    
        return self
    
    def span_name(self, span_name: str) -> typing.Self:    
        """
        @deprecated Query traces by span name
        """
            
        self._internal.span_name = span_name
    
        return self
    
    def min_duration(self, min_duration: str) -> typing.Self:    
        """
        @deprecated Define the minimum duration to select traces. Use duration format, for example: 1.2s, 100ms
        """
            
        self._internal.min_duration = min_duration
    
        return self
    
    def max_duration(self, max_duration: str) -> typing.Self:    
        """
        @deprecated Define the maximum duration to select traces. Use duration format, for example: 1.2s, 100ms
        """
            
        self._internal.max_duration = max_duration
    
        return self
    
    def service_map_query(self, service_map_query: str) -> typing.Self:    
        """
        Filters to be included in a PromQL query to select data for the service graph. Example: {client="app",service="app"}
        """
            
        self._internal.service_map_query = service_map_query
    
        return self
    
    def service_map_include_namespace(self, service_map_include_namespace: bool) -> typing.Self:    
        """
        Use service.namespace in addition to service.name to uniquely identify a service.
        """
            
        self._internal.service_map_include_namespace = service_map_include_namespace
    
        return self
    
    def limit(self, limit: int) -> typing.Self:    
        """
        Defines the maximum number of traces that are returned from Tempo
        """
            
        self._internal.limit = limit
    
        return self
    
    def spss(self, spss: int) -> typing.Self:    
        """
        Defines the maximum number of spans per spanset that are returned from Tempo
        """
            
        self._internal.spss = spss
    
        return self
    
    def filters(self, filters: list[cogbuilder.Builder[tempo.TraceqlFilter]]) -> typing.Self:    
        filters_resources = [r1.build() for r1 in filters]
        self._internal.filters = filters_resources
    
        return self
    
    def group_by(self, group_by: list[cogbuilder.Builder[tempo.TraceqlFilter]]) -> typing.Self:    
        """
        Filters that are used to query the metrics summary
        """
            
        group_by_resources = [r1.build() for r1 in group_by]
        self._internal.group_by = group_by_resources
    
        return self
    
    def datasource(self, datasource: dashboard.DataSourceRef) -> typing.Self:    
        """
        For mixed data sources the selected datasource is on the query level.
        For non mixed scenarios this is undefined.
        TODO find a better way to do this ^ that's friendly to schema
        TODO this shouldn't be unknown but DataSourceRef | null
        """
            
        self._internal.datasource = datasource
    
        return self
    
    def table_type(self, table_type: tempo.SearchTableType) -> typing.Self:    
        """
        The type of the table that is used to display the search results
        """
            
        self._internal.table_type = table_type
    
        return self
    


class TraceqlFilter(cogbuilder.Builder[tempo.TraceqlFilter]):
    _internal: tempo.TraceqlFilter

    def __init__(self):
        self._internal = tempo.TraceqlFilter()

    def build(self) -> tempo.TraceqlFilter:
        """
        Builds the object.
        """
        return self._internal    
    
    def id(self, id_val: str) -> typing.Self:    
        """
        Uniquely identify the filter, will not be used in the query generation
        """
            
        self._internal.id_val = id_val
    
        return self
    
    def tag(self, tag: str) -> typing.Self:    
        """
        The tag for the search filter, for example: .http.status_code, .service.name, status
        """
            
        self._internal.tag = tag
    
        return self
    
    def operator(self, operator: str) -> typing.Self:    
        """
        The operator that connects the tag to the value, for example: =, >, !=, =~
        """
            
        self._internal.operator = operator
    
        return self
    
    def value(self, value: typing.Union[str, list[str]]) -> typing.Self:    
        """
        The value for the search filter
        """
            
        self._internal.value = value
    
        return self
    
    def value_type(self, value_type: str) -> typing.Self:    
        """
        The type of the value, used for example to check whether we need to wrap the value in quotes when generating the query
        """
            
        self._internal.value_type = value_type
    
        return self
    
    def scope(self, scope: tempo.TraceqlSearchScope) -> typing.Self:    
        """
        The scope of the filter, can either be unscoped/all scopes, resource or span
        """
            
        self._internal.scope = scope
    
        return self
    



# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
from ..cog import builder as cogbuilder
from ..models import prometheus


class Dataquery(cogbuilder.Builder[prometheus.Dataquery]):
    _internal: prometheus.Dataquery

    def __init__(self):
        self._internal = prometheus.Dataquery()

    def build(self) -> prometheus.Dataquery:
        """
        Builds the object.
        """
        return self._internal    
    
    def expr(self, expr: str) -> typing.Self:    
        """
        The actual expression/query that will be evaluated by Prometheus
        """
            
        self._internal.expr = expr
    
        return self
    
    def instant(self) -> typing.Self:    
        """
        Returns only the latest value that Prometheus has scraped for the requested time series
        """
            
        self._internal.instant = True    
        self._internal.range_val = False
    
        return self
    
    def range(self) -> typing.Self:    
        """
        Returns a Range vector, comprised of a set of time series containing a range of data points over time for each time series
        """
            
        self._internal.range_val = True    
        self._internal.instant = False
    
        return self
    
    def exemplar(self, exemplar: bool) -> typing.Self:    
        """
        Execute an additional query to identify interesting raw samples relevant for the given expr
        """
            
        self._internal.exemplar = exemplar
    
        return self
    
    def editor_mode(self, editor_mode: prometheus.QueryEditorMode) -> typing.Self:    
        """
        Specifies which editor is being used to prepare the query. It can be "code" or "builder"
        """
            
        self._internal.editor_mode = editor_mode
    
        return self
    
    def format(self, format_val: prometheus.PromQueryFormat) -> typing.Self:    
        """
        Query format to determine how to display data points in panel. It can be "time_series", "table", "heatmap"
        """
            
        self._internal.format_val = format_val
    
        return self
    
    def legend_format(self, legend_format: str) -> typing.Self:    
        """
        Series name override or template. Ex. {{hostname}} will be replaced with label value for hostname
        """
            
        self._internal.legend_format = legend_format
    
        return self
    
    def interval_factor(self, interval_factor: float) -> typing.Self:    
        """
        @deprecated Used to specify how many times to divide max data points by. We use max data points under query options
        See https://github.com/grafana/grafana/issues/48081
        """
            
        self._internal.interval_factor = interval_factor
    
        return self
    
    def ref_id(self, ref_id: str) -> typing.Self:    
        """
        A unique identifier for the query within the list of targets.
        In server side expressions, the refId is used as a variable name to identify results.
        By default, the UI will assign A->Z; however setting meaningful names may be useful.
        """
            
        self._internal.ref_id = ref_id
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        """
        true if query is disabled (ie should not be returned to the dashboard)
        Note this does not always imply that the query should not be executed since
        the results from a hidden query may be used as the input to other queries (SSE etc)
        """
            
        self._internal.hide = hide
    
        return self
    
    def query_type(self, query_type: str) -> typing.Self:    
        """
        Specify the query flavor
        TODO make this required and give it a default
        """
            
        self._internal.query_type = query_type
    
        return self
    
    def datasource(self, datasource: object) -> typing.Self:    
        """
        For mixed data sources the selected datasource is on the query level.
        For non mixed scenarios this is undefined.
        TODO find a better way to do this ^ that's friendly to schema
        TODO this shouldn't be unknown but DataSourceRef | null
        """
            
        self._internal.datasource = datasource
    
        return self
    
    def interval(self, interval: str) -> typing.Self:    
        """
        An additional lower limit for the step parameter of the Prometheus query and for the
        `$__interval` and `$__rate_interval` variables.
        """
            
        self._internal.interval = interval
    
        return self
    
    def range_and_instant(self) -> typing.Self:    
        self._internal.range_val = True    
        self._internal.instant = True
    
        return self
    



# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing


class Folder:
    """
    TODO:
    common metadata will soon support setting the parent folder in the metadata
    """

    # Unique folder id. (will be k8s name)
    uid: str
    # Folder title
    title: str
    # Description of the folder.
    description: typing.Optional[str]

    def __init__(self, uid: str = "", title: str = "", description: typing.Optional[str] = None):
        self.uid = uid
        self.title = title
        self.description = description

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "uid": self.uid,
            "title": self.title,
        }
        if self.description is not None:
            payload["description"] = self.description
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "uid" in data:
            args["uid"] = data["uid"]
        if "title" in data:
            args["title"] = data["title"]
        if "description" in data:
            args["description"] = data["description"]        

        return cls(**args)



# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
from ..cog import builder as cogbuilder
from ..models import heatmap
from ..models import common
from ..models import dashboard
from ..cog import variants as cogvariants


class HeatmapColorOptions(cogbuilder.Builder[heatmap.HeatmapColorOptions]):    
    """
    Controls various color options
    """
    
    _internal: heatmap.HeatmapColorOptions

    def __init__(self):
        self._internal = heatmap.HeatmapColorOptions()

    def build(self) -> heatmap.HeatmapColorOptions:
        """
        Builds the object.
        """
        return self._internal    
    
    def mode(self, mode: heatmap.HeatmapColorMode) -> typing.Self:    
        """
        Sets the color mode
        """
            
        self._internal.mode = mode
    
        return self
    
    def scheme(self, scheme: str) -> typing.Self:    
        """
        Controls the color scheme used
        """
            
        self._internal.scheme = scheme
    
        return self
    
    def fill(self, fill: str) -> typing.Self:    
        """
        Controls the color fill when in opacity mode
        """
            
        self._internal.fill = fill
    
        return self
    
    def scale(self, scale: heatmap.HeatmapColorScale) -> typing.Self:    
        """
        Controls the color scale
        """
            
        self._internal.scale = scale
    
        return self
    
    def exponent(self, exponent: float) -> typing.Self:    
        """
        Controls the exponent when scale is set to exponential
        """
            
        self._internal.exponent = exponent
    
        return self
    
    def steps(self, steps: int) -> typing.Self:    
        """
        Controls the number of color steps
        """
            
        if not steps >= 2:
            raise ValueError("steps must be >= 2")
        if not steps <= 128:
            raise ValueError("steps must be <= 128")
        self._internal.steps = steps
    
        return self
    
    def reverse(self, reverse: bool) -> typing.Self:    
        """
        Reverses the color scheme
        """
            
        self._internal.reverse = reverse
    
        return self
    
    def min(self, min_val: float) -> typing.Self:    
        """
        Sets the minimum value for the color scale
        """
            
        self._internal.min_val = min_val
    
        return self
    
    def max(self, max_val: float) -> typing.Self:    
        """
        Sets the maximum value for the color scale
        """
            
        self._internal.max_val = max_val
    
        return self
    


class YAxisConfig(cogbuilder.Builder[heatmap.YAxisConfig]):    
    """
    Configuration options for the yAxis
    """
    
    _internal: heatmap.YAxisConfig

    def __init__(self):
        self._internal = heatmap.YAxisConfig()

    def build(self) -> heatmap.YAxisConfig:
        """
        Builds the object.
        """
        return self._internal    
    
    def unit(self, unit: str) -> typing.Self:    
        """
        Sets the yAxis unit
        """
            
        self._internal.unit = unit
    
        return self
    
    def reverse(self, reverse: bool) -> typing.Self:    
        """
        Reverses the yAxis
        """
            
        self._internal.reverse = reverse
    
        return self
    
    def decimals(self, decimals: float) -> typing.Self:    
        """
        Controls the number of decimals for yAxis values
        """
            
        self._internal.decimals = decimals
    
        return self
    
    def min(self, min_val: float) -> typing.Self:    
        """
        Sets the minimum value for the yAxis
        """
            
        self._internal.min_val = min_val
    
        return self
    
    def axis_placement(self, axis_placement: common.AxisPlacement) -> typing.Self:    
        self._internal.axis_placement = axis_placement
    
        return self
    
    def axis_color_mode(self, axis_color_mode: common.AxisColorMode) -> typing.Self:    
        self._internal.axis_color_mode = axis_color_mode
    
        return self
    
    def axis_label(self, axis_label: str) -> typing.Self:    
        self._internal.axis_label = axis_label
    
        return self
    
    def axis_width(self, axis_width: float) -> typing.Self:    
        self._internal.axis_width = axis_width
    
        return self
    
    def axis_soft_min(self, axis_soft_min: float) -> typing.Self:    
        self._internal.axis_soft_min = axis_soft_min
    
        return self
    
    def axis_soft_max(self, axis_soft_max: float) -> typing.Self:    
        self._internal.axis_soft_max = axis_soft_max
    
        return self
    
    def axis_grid_show(self, axis_grid_show: bool) -> typing.Self:    
        self._internal.axis_grid_show = axis_grid_show
    
        return self
    
    def scale_distribution(self, scale_distribution: cogbuilder.Builder[common.ScaleDistributionConfig]) -> typing.Self:    
        scale_distribution_resource = scale_distribution.build()
        self._internal.scale_distribution = scale_distribution_resource
    
        return self
    
    def axis_centered_zero(self, axis_centered_zero: bool) -> typing.Self:    
        self._internal.axis_centered_zero = axis_centered_zero
    
        return self
    
    def max(self, max_val: float) -> typing.Self:    
        """
        Sets the maximum value for the yAxis
        """
            
        self._internal.max_val = max_val
    
        return self
    
    def axis_border_show(self, axis_border_show: bool) -> typing.Self:    
        self._internal.axis_border_show = axis_border_show
    
        return self
    


class CellValues(cogbuilder.Builder[heatmap.CellValues]):    
    """
    Controls cell value options
    """
    
    _internal: heatmap.CellValues

    def __init__(self):
        self._internal = heatmap.CellValues()

    def build(self) -> heatmap.CellValues:
        """
        Builds the object.
        """
        return self._internal    
    
    def unit(self, unit: str) -> typing.Self:    
        """
        Controls the cell value unit
        """
            
        self._internal.unit = unit
    
        return self
    
    def decimals(self, decimals: float) -> typing.Self:    
        """
        Controls the number of decimals for cell values
        """
            
        self._internal.decimals = decimals
    
        return self
    


class FilterValueRange(cogbuilder.Builder[heatmap.FilterValueRange]):    
    """
    Controls the value filter range
    """
    
    _internal: heatmap.FilterValueRange

    def __init__(self):
        self._internal = heatmap.FilterValueRange()

    def build(self) -> heatmap.FilterValueRange:
        """
        Builds the object.
        """
        return self._internal    
    
    def le(self, le: float) -> typing.Self:    
        """
        Sets the filter range to values less than or equal to the given value
        """
            
        self._internal.le = le
    
        return self
    
    def ge(self, ge: float) -> typing.Self:    
        """
        Sets the filter range to values greater than or equal to the given value
        """
            
        self._internal.ge = ge
    
        return self
    


class HeatmapTooltip(cogbuilder.Builder[heatmap.HeatmapTooltip]):    
    """
    Controls tooltip options
    """
    
    _internal: heatmap.HeatmapTooltip

    def __init__(self):
        self._internal = heatmap.HeatmapTooltip()

    def build(self) -> heatmap.HeatmapTooltip:
        """
        Builds the object.
        """
        return self._internal    
    
    def show(self, show: bool) -> typing.Self:    
        """
        Controls if the tooltip is shown
        """
            
        self._internal.show = show
    
        return self
    
    def y_histogram(self, y_histogram: bool) -> typing.Self:    
        """
        Controls if the tooltip shows a histogram of the y-axis values
        """
            
        self._internal.y_histogram = y_histogram
    
        return self
    
    def show_color_scale(self, show_color_scale: bool) -> typing.Self:    
        """
        Controls if the tooltip shows a color scale in header
        """
            
        self._internal.show_color_scale = show_color_scale
    
        return self
    


class HeatmapLegend(cogbuilder.Builder[heatmap.HeatmapLegend]):    
    """
    Controls legend options
    """
    
    _internal: heatmap.HeatmapLegend

    def __init__(self):
        self._internal = heatmap.HeatmapLegend()

    def build(self) -> heatmap.HeatmapLegend:
        """
        Builds the object.
        """
        return self._internal    
    
    def show(self, show: bool) -> typing.Self:    
        """
        Controls if the legend is shown
        """
            
        self._internal.show = show
    
        return self
    


class ExemplarConfig(cogbuilder.Builder[heatmap.ExemplarConfig]):    
    """
    Controls exemplar options
    """
    
    _internal: heatmap.ExemplarConfig

    def __init__(self):
        self._internal = heatmap.ExemplarConfig()

    def build(self) -> heatmap.ExemplarConfig:
        """
        Builds the object.
        """
        return self._internal    
    
    def color(self, color: str) -> typing.Self:    
        """
        Sets the color of the exemplar markers
        """
            
        self._internal.color = color
    
        return self
    


class RowsHeatmapOptions(cogbuilder.Builder[heatmap.RowsHeatmapOptions]):    
    """
    Controls frame rows options
    """
    
    _internal: heatmap.RowsHeatmapOptions

    def __init__(self):
        self._internal = heatmap.RowsHeatmapOptions()

    def build(self) -> heatmap.RowsHeatmapOptions:
        """
        Builds the object.
        """
        return self._internal    
    
    def value(self, value: str) -> typing.Self:    
        """
        Sets the name of the cell when not calculating from data
        """
            
        self._internal.value = value
    
        return self
    
    def layout(self, layout: common.HeatmapCellLayout) -> typing.Self:    
        """
        Controls tick alignment when not calculating from data
        """
            
        self._internal.layout = layout
    
        return self
    


class Panel(cogbuilder.Builder[dashboard.Panel]):    
    """
    Dashboard panels are the basic visualization building blocks.
    """
    
    _internal: dashboard.Panel

    def __init__(self):
        self._internal = dashboard.Panel()        
        self._internal.type_val = "heatmap"

    def build(self) -> dashboard.Panel:
        """
        Builds the object.
        """
        return self._internal    
    
    def id(self, id_val: int) -> typing.Self:    
        """
        Unique identifier of the panel. Generated by Grafana when creating a new panel. It must be unique within a dashboard, but not globally.
        """
            
        self._internal.id_val = id_val
    
        return self
    
    def targets(self, targets: list[cogbuilder.Builder[cogvariants.Dataquery]]) -> typing.Self:    
        """
        Depends on the panel plugin. See the plugin documentation for details.
        """
            
        targets_resources = [r1.build() for r1 in targets]
        self._internal.targets = targets_resources
    
        return self
    
    def with_target(self, target: cogbuilder.Builder[cogvariants.Dataquery]) -> typing.Self:    
        """
        Depends on the panel plugin. See the plugin documentation for details.
        """
            
        if self._internal.targets is None:
            self._internal.targets = []
        
        target_resource = target.build()
        self._internal.targets.append(target_resource)
    
        return self
    
    def title(self, title: str) -> typing.Self:    
        """
        Panel title.
        """
            
        self._internal.title = title
    
        return self
    
    def description(self, description: str) -> typing.Self:    
        """
        Panel description.
        """
            
        self._internal.description = description
    
        return self
    
    def transparent(self, transparent: bool) -> typing.Self:    
        """
        Whether to display the panel without a background.
        """
            
        self._internal.transparent = transparent
    
        return self
    
    def datasource(self, datasource: dashboard.DataSourceRef) -> typing.Self:    
        """
        The datasource used in all targets.
        """
            
        self._internal.datasource = datasource
    
        return self
    
    def grid_pos(self, grid_pos: dashboard.GridPos) -> typing.Self:    
        """
        Grid position.
        """
            
        self._internal.grid_pos = grid_pos
    
        return self
    
    def height(self, h: int) -> typing.Self:    
        """
        Panel height. The height is the number of rows from the top edge of the panel.
        """
            
        if not h > 0:
            raise ValueError("h must be > 0")
        if self._internal.grid_pos is None:
            self._internal.grid_pos = dashboard.GridPos()
        assert isinstance(self._internal.grid_pos, dashboard.GridPos)
        self._internal.grid_pos.h = h
    
        return self
    
    def span(self, w: int) -> typing.Self:    
        """
        Panel width. The width is the number of columns from the left edge of the panel.
        """
            
        if not w > 0:
            raise ValueError("w must be > 0")
        if not w <= 24:
            raise ValueError("w must be <= 24")
        if self._internal.grid_pos is None:
            self._internal.grid_pos = dashboard.GridPos()
        assert isinstance(self._internal.grid_pos, dashboard.GridPos)
        self._internal.grid_pos.w = w
    
        return self
    
    def links(self, links: list[cogbuilder.Builder[dashboard.DashboardLink]]) -> typing.Self:    
        """
        Panel links.
        """
            
        links_resources = [r1.build() for r1 in links]
        self._internal.links = links_resources
    
        return self
    
    def repeat(self, repeat: str) -> typing.Self:    
        """
        Name of template variable to repeat for.
        """
            
        self._internal.repeat = repeat
    
        return self
    
    def repeat_direction(self, repeat_direction: typing.Literal["h", "v"]) -> typing.Self:    
        """
        Direction to repeat in if 'repeat' is set.
        `h` for horizontal, `v` for vertical.
        """
            
        self._internal.repeat_direction = repeat_direction
    
        return self
    
    def max_per_row(self, max_per_row: float) -> typing.Self:    
        """
        Option for repeated panels that controls max items per row
        Only relevant for horizontally repeated panels
        """
            
        self._internal.max_per_row = max_per_row
    
        return self
    
    def max_data_points(self, max_data_points: float) -> typing.Self:    
        """
        The maximum number of data points that the panel queries are retrieving.
        """
            
        self._internal.max_data_points = max_data_points
    
        return self
    
    def transformations(self, transformations: list[dashboard.DataTransformerConfig]) -> typing.Self:    
        """
        List of transformations that are applied to the panel data before rendering.
        When there are multiple transformations, Grafana applies them in the order they are listed.
        Each transformation creates a result set that then passes on to the next transformation in the processing pipeline.
        """
            
        self._internal.transformations = transformations
    
        return self
    
    def with_transformation(self, transformation: dashboard.DataTransformerConfig) -> typing.Self:    
        """
        List of transformations that are applied to the panel data before rendering.
        When there are multiple transformations, Grafana applies them in the order they are listed.
        Each transformation creates a result set that then passes on to the next transformation in the processing pipeline.
        """
            
        if self._internal.transformations is None:
            self._internal.transformations = []
        
        self._internal.transformations.append(transformation)
    
        return self
    
    def interval(self, interval: str) -> typing.Self:    
        """
        The min time interval setting defines a lower limit for the $__interval and $__interval_ms variables.
        This value must be formatted as a number followed by a valid time
        identifier like: "40s", "3d", etc.
        See: https://grafana.com/docs/grafana/latest/panels-visualizations/query-transform-data/#query-options
        """
            
        self._internal.interval = interval
    
        return self
    
    def time_from(self, time_from: str) -> typing.Self:    
        """
        Overrides the relative time range for individual panels,
        which causes them to be different than what is selected in
        the dashboard time picker in the top-right corner of the dashboard. You can use this to show metrics from different
        time periods or days on the same dashboard.
        The value is formatted as time operation like: `now-5m` (Last 5 minutes), `now/d` (the day so far),
        `now-5d/d`(Last 5 days), `now/w` (This week so far), `now-2y/y` (Last 2 years).
        Note: Panel time overrides have no effect when the dashboard’s time range is absolute.
        See: https://grafana.com/docs/grafana/latest/panels-visualizations/query-transform-data/#query-options
        """
            
        self._internal.time_from = time_from
    
        return self
    
    def time_shift(self, time_shift: str) -> typing.Self:    
        """
        Overrides the time range for individual panels by shifting its start and end relative to the time picker.
        For example, you can shift the time range for the panel to be two hours earlier than the dashboard time picker setting `2h`.
        Note: Panel time overrides have no effect when the dashboard’s time range is absolute.
        See: https://grafana.com/docs/grafana/latest/panels-visualizations/query-transform-data/#query-options
        """
            
        self._internal.time_shift = time_shift
    
        return self
    
    def hide_time_override(self, hide_time_override: bool) -> typing.Self:    
        """
        Controls if the timeFrom or timeShift overrides are shown in the panel header
        """
            
        self._internal.hide_time_override = hide_time_override
    
        return self
    
    def library_panel(self, library_panel: dashboard.LibraryPanelRef) -> typing.Self:    
        """
        Dynamically load the panel
        """
            
        self._internal.library_panel = library_panel
    
        return self
    
    def display_name(self, display_name: str) -> typing.Self:    
        """
        The display value for this field.  This supports template variables blank is auto
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        self._internal.field_config.defaults.display_name = display_name
    
        return self
    
    def unit(self, unit: str) -> typing.Self:    
        """
        Unit a field should use. The unit you select is applied to all fields except time.
        You can use the units ID availables in Grafana or a custom unit.
        Available units in Grafana: https://github.com/grafana/grafana/blob/main/packages/grafana-data/src/valueFormats/categories.ts
        As custom unit, you can use the following formats:
        `suffix:<suffix>` for custom unit that should go after value.
        `prefix:<prefix>` for custom unit that should go before value.
        `time:<format>` For custom date time formats type for example `time:YYYY-MM-DD`.
        `si:<base scale><unit characters>` for custom SI units. For example: `si: mF`. This one is a bit more advanced as you can specify both a unit and the source data scale. So if your source data is represented as milli (thousands of) something prefix the unit with that SI scale character.
        `count:<unit>` for a custom count unit.
        `currency:<unit>` for custom a currency unit.
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        self._internal.field_config.defaults.unit = unit
    
        return self
    
    def decimals(self, decimals: float) -> typing.Self:    
        """
        Specify the number of decimals Grafana includes in the rendered value.
        If you leave this field blank, Grafana automatically truncates the number of decimals based on the value.
        For example 1.1234 will display as 1.12 and 100.456 will display as 100.
        To display all decimals, set the unit to `String`.
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        self._internal.field_config.defaults.decimals = decimals
    
        return self
    
    def min(self, min_val: float) -> typing.Self:    
        """
        The minimum value used in percentage threshold calculations. Leave blank for auto calculation based on all series and fields.
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        self._internal.field_config.defaults.min_val = min_val
    
        return self
    
    def max(self, max_val: float) -> typing.Self:    
        """
        The maximum value used in percentage threshold calculations. Leave blank for auto calculation based on all series and fields.
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        self._internal.field_config.defaults.max_val = max_val
    
        return self
    
    def mappings(self, mappings: list[dashboard.ValueMapping]) -> typing.Self:    
        """
        Convert input values into a display string
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        self._internal.field_config.defaults.mappings = mappings
    
        return self
    
    def thresholds(self, thresholds: cogbuilder.Builder[dashboard.ThresholdsConfig]) -> typing.Self:    
        """
        Map numeric values to states
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        thresholds_resource = thresholds.build()
        self._internal.field_config.defaults.thresholds = thresholds_resource
    
        return self
    
    def color_scheme(self, color: cogbuilder.Builder[dashboard.FieldColor]) -> typing.Self:    
        """
        Panel color configuration
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        color_resource = color.build()
        self._internal.field_config.defaults.color = color_resource
    
        return self
    
    def data_links(self, links: list[cogbuilder.Builder[dashboard.DashboardLink]]) -> typing.Self:    
        """
        The behavior when clicking on a result
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        links_resources = [r1.build() for r1 in links]
        self._internal.field_config.defaults.links = links_resources
    
        return self
    
    def no_value(self, no_value: str) -> typing.Self:    
        """
        Alternative to empty string
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        self._internal.field_config.defaults.no_value = no_value
    
        return self
    
    def overrides(self, overrides: list[cogbuilder.Builder[dashboard.DashboardFieldConfigSourceOverrides]]) -> typing.Self:    
        """
        Overrides are the options applied to specific fields overriding the defaults.
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        overrides_resources = [r1.build() for r1 in overrides]
        self._internal.field_config.overrides = overrides_resources
    
        return self
    
    def with_override(self, matcher: dashboard.MatcherConfig, properties: list[dashboard.DynamicConfigValue]) -> typing.Self:    
        """
        Overrides are the options applied to specific fields overriding the defaults.
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.overrides is None:
            self._internal.field_config.overrides = []
        
        self._internal.field_config.overrides.append(dashboard.DashboardFieldConfigSourceOverrides(
            matcher=matcher,
            properties=properties,
        ))
    
        return self
    
    def override_by_name(self, name: str, properties: list[dashboard.DynamicConfigValue]) -> typing.Self:    
        """
        Adds override rules for a specific field, referred to by its name.
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.overrides is None:
            self._internal.field_config.overrides = []
        
        self._internal.field_config.overrides.append(dashboard.DashboardFieldConfigSourceOverrides(
            matcher=dashboard.MatcherConfig(
            id_val="byName",
            options=name,
        ),
            properties=properties,
        ))
    
        return self
    
    def override_by_regexp(self, regexp: str, properties: list[dashboard.DynamicConfigValue]) -> typing.Self:    
        """
        Adds override rules for the fields whose name match the given regexp.
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.overrides is None:
            self._internal.field_config.overrides = []
        
        self._internal.field_config.overrides.append(dashboard.DashboardFieldConfigSourceOverrides(
            matcher=dashboard.MatcherConfig(
            id_val="byRegexp",
            options=regexp,
        ),
            properties=properties,
        ))
    
        return self
    
    def override_by_field_type(self, field_type: str, properties: list[dashboard.DynamicConfigValue]) -> typing.Self:    
        """
        Adds override rules for all the fields of the given type.
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.overrides is None:
            self._internal.field_config.overrides = []
        
        self._internal.field_config.overrides.append(dashboard.DashboardFieldConfigSourceOverrides(
            matcher=dashboard.MatcherConfig(
            id_val="byType",
            options=field_type,
        ),
            properties=properties,
        ))
    
        return self
    
    def override_by_query(self, query_ref_id: str, properties: list[dashboard.DynamicConfigValue]) -> typing.Self:    
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.overrides is None:
            self._internal.field_config.overrides = []
        
        self._internal.field_config.overrides.append(dashboard.DashboardFieldConfigSourceOverrides(
            matcher=dashboard.MatcherConfig(
            id_val="byFrameRefID",
            options=query_ref_id,
        ),
            properties=properties,
        ))
    
        return self
    
    def calculate(self, calculate: bool) -> typing.Self:    
        """
        Controls if the heatmap should be calculated from data
        """
            
        if self._internal.options is None:
            self._internal.options = heatmap.Options()
        assert isinstance(self._internal.options, heatmap.Options)
        self._internal.options.calculate = calculate
    
        return self
    
    def calculation(self, calculation: cogbuilder.Builder[common.HeatmapCalculationOptions]) -> typing.Self:    
        """
        Calculation options for the heatmap
        """
            
        if self._internal.options is None:
            self._internal.options = heatmap.Options()
        assert isinstance(self._internal.options, heatmap.Options)
        calculation_resource = calculation.build()
        self._internal.options.calculation = calculation_resource
    
        return self
    
    def color(self, color: cogbuilder.Builder[heatmap.HeatmapColorOptions]) -> typing.Self:    
        """
        Controls the color options
        """
            
        if self._internal.options is None:
            self._internal.options = heatmap.Options()
        assert isinstance(self._internal.options, heatmap.Options)
        color_resource = color.build()
        self._internal.options.color = color_resource
    
        return self
    
    def filter_values(self, filter_values: cogbuilder.Builder[heatmap.FilterValueRange]) -> typing.Self:    
        """
        Filters values between a given range
        """
            
        if self._internal.options is None:
            self._internal.options = heatmap.Options()
        assert isinstance(self._internal.options, heatmap.Options)
        filter_values_resource = filter_values.build()
        self._internal.options.filter_values = filter_values_resource
    
        return self
    
    def rows_frame(self, rows_frame: cogbuilder.Builder[heatmap.RowsHeatmapOptions]) -> typing.Self:    
        """
        Controls tick alignment and value name when not calculating from data
        """
            
        if self._internal.options is None:
            self._internal.options = heatmap.Options()
        assert isinstance(self._internal.options, heatmap.Options)
        rows_frame_resource = rows_frame.build()
        self._internal.options.rows_frame = rows_frame_resource
    
        return self
    
    def show_value(self, show_value: common.VisibilityMode) -> typing.Self:    
        """
        | *{
        	layout: ui.HeatmapCellLayout & "auto" // TODO: fix after remove when https://github.com/grafana/cuetsy/issues/74 is fixed
        }
        Controls the display of the value in the cell
        """
            
        if self._internal.options is None:
            self._internal.options = heatmap.Options()
        assert isinstance(self._internal.options, heatmap.Options)
        self._internal.options.show_value = show_value
    
        return self
    
    def cell_gap(self, cell_gap: int) -> typing.Self:    
        """
        Controls gap between cells
        """
            
        if not cell_gap <= 25:
            raise ValueError("cell_gap must be <= 25")
        if self._internal.options is None:
            self._internal.options = heatmap.Options()
        assert isinstance(self._internal.options, heatmap.Options)
        self._internal.options.cell_gap = cell_gap
    
        return self
    
    def cell_radius(self, cell_radius: float) -> typing.Self:    
        """
        Controls cell radius
        """
            
        if self._internal.options is None:
            self._internal.options = heatmap.Options()
        assert isinstance(self._internal.options, heatmap.Options)
        self._internal.options.cell_radius = cell_radius
    
        return self
    
    def cell_values(self, cell_values: cogbuilder.Builder[heatmap.CellValues]) -> typing.Self:    
        """
        Controls cell value unit
        """
            
        if self._internal.options is None:
            self._internal.options = heatmap.Options()
        assert isinstance(self._internal.options, heatmap.Options)
        cell_values_resource = cell_values.build()
        self._internal.options.cell_values = cell_values_resource
    
        return self
    
    def y_axis(self, y_axis: cogbuilder.Builder[heatmap.YAxisConfig]) -> typing.Self:    
        """
        Controls yAxis placement
        """
            
        if self._internal.options is None:
            self._internal.options = heatmap.Options()
        assert isinstance(self._internal.options, heatmap.Options)
        y_axis_resource = y_axis.build()
        self._internal.options.y_axis = y_axis_resource
    
        return self
    
    def show_legend(self) -> typing.Self:    
        """
        | *{
        	axisPlacement: ui.AxisPlacement & "left" // TODO: fix after remove when https://github.com/grafana/cuetsy/issues/74 is fixed
        }
        Controls legend options
        """
            
        if self._internal.options is None:
            self._internal.options = heatmap.Options()
        assert isinstance(self._internal.options, heatmap.Options)
        if self._internal.options.legend is None:
            self._internal.options.legend = heatmap.HeatmapLegend()
        assert isinstance(self._internal.options.legend, heatmap.HeatmapLegend)
        self._internal.options.legend.show = True
    
        return self
    
    def hide_legend(self) -> typing.Self:    
        """
        | *{
        	axisPlacement: ui.AxisPlacement & "left" // TODO: fix after remove when https://github.com/grafana/cuetsy/issues/74 is fixed
        }
        Controls legend options
        """
            
        if self._internal.options is None:
            self._internal.options = heatmap.Options()
        assert isinstance(self._internal.options, heatmap.Options)
        if self._internal.options.legend is None:
            self._internal.options.legend = heatmap.HeatmapLegend()
        assert isinstance(self._internal.options.legend, heatmap.HeatmapLegend)
        self._internal.options.legend.show = False
    
        return self
    
    def show_tooltip(self) -> typing.Self:    
        """
        Controls if the tooltip is shown
        """
            
        if self._internal.options is None:
            self._internal.options = heatmap.Options()
        assert isinstance(self._internal.options, heatmap.Options)
        if self._internal.options.tooltip is None:
            self._internal.options.tooltip = heatmap.HeatmapTooltip()
        assert isinstance(self._internal.options.tooltip, heatmap.HeatmapTooltip)
        self._internal.options.tooltip.show = True
    
        return self
    
    def hide_tooltip(self) -> typing.Self:    
        """
        Controls if the tooltip is shown
        """
            
        if self._internal.options is None:
            self._internal.options = heatmap.Options()
        assert isinstance(self._internal.options, heatmap.Options)
        if self._internal.options.tooltip is None:
            self._internal.options.tooltip = heatmap.HeatmapTooltip()
        assert isinstance(self._internal.options.tooltip, heatmap.HeatmapTooltip)
        self._internal.options.tooltip.show = False
    
        return self
    
    def show_y_histogram(self) -> typing.Self:    
        """
        Controls if the tooltip shows a histogram of the y-axis values
        """
            
        if self._internal.options is None:
            self._internal.options = heatmap.Options()
        assert isinstance(self._internal.options, heatmap.Options)
        if self._internal.options.tooltip is None:
            self._internal.options.tooltip = heatmap.HeatmapTooltip()
        assert isinstance(self._internal.options.tooltip, heatmap.HeatmapTooltip)
        self._internal.options.tooltip.y_histogram = True
    
        return self
    
    def hide_y_histogram(self) -> typing.Self:    
        """
        Controls if the tooltip shows a histogram of the y-axis values
        """
            
        if self._internal.options is None:
            self._internal.options = heatmap.Options()
        assert isinstance(self._internal.options, heatmap.Options)
        if self._internal.options.tooltip is None:
            self._internal.options.tooltip = heatmap.HeatmapTooltip()
        assert isinstance(self._internal.options.tooltip, heatmap.HeatmapTooltip)
        self._internal.options.tooltip.y_histogram = False
    
        return self
    
    def show_color_scale(self, show_color_scale: bool) -> typing.Self:    
        """
        Controls if the tooltip shows a color scale in header
        """
            
        if self._internal.options is None:
            self._internal.options = heatmap.Options()
        assert isinstance(self._internal.options, heatmap.Options)
        if self._internal.options.tooltip is None:
            self._internal.options.tooltip = heatmap.HeatmapTooltip()
        assert isinstance(self._internal.options.tooltip, heatmap.HeatmapTooltip)
        self._internal.options.tooltip.show_color_scale = show_color_scale
    
        return self
    
    def exemplars_color(self, color: str) -> typing.Self:    
        """
        Controls exemplar options
        """
            
        if self._internal.options is None:
            self._internal.options = heatmap.Options()
        assert isinstance(self._internal.options, heatmap.Options)
        if self._internal.options.exemplars is None:
            self._internal.options.exemplars = heatmap.ExemplarConfig()
        assert isinstance(self._internal.options.exemplars, heatmap.ExemplarConfig)
        self._internal.options.exemplars.color = color
    
        return self
    
    def scale_distribution(self, scale_distribution: cogbuilder.Builder[common.ScaleDistributionConfig]) -> typing.Self:    
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        if self._internal.field_config.defaults.custom is None:
            self._internal.field_config.defaults.custom = heatmap.FieldConfig()
        assert isinstance(self._internal.field_config.defaults.custom, heatmap.FieldConfig)
        scale_distribution_resource = scale_distribution.build()
        self._internal.field_config.defaults.custom.scale_distribution = scale_distribution_resource
    
        return self
    
    def hide_from(self, hide_from: cogbuilder.Builder[common.HideSeriesConfig]) -> typing.Self:    
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        if self._internal.field_config.defaults.custom is None:
            self._internal.field_config.defaults.custom = heatmap.FieldConfig()
        assert isinstance(self._internal.field_config.defaults.custom, heatmap.FieldConfig)
        hide_from_resource = hide_from.build()
        self._internal.field_config.defaults.custom.hide_from = hide_from_resource
    
        return self
    



# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
from ..cog import builder as cogbuilder
from ..models import role


class Role(cogbuilder.Builder[role.Role]):
    _internal: role.Role

    def __init__(self):
        self._internal = role.Role()

    def build(self) -> role.Role:
        """
        Builds the object.
        """
        return self._internal    
    
    def name(self, name: str) -> typing.Self:    
        """
        The role identifier `managed:builtins:editor:permissions`
        """
            
        self._internal.name = name
    
        return self
    
    def display_name(self, display_name: str) -> typing.Self:    
        """
        Optional display
        """
            
        self._internal.display_name = display_name
    
        return self
    
    def group_name(self, group_name: str) -> typing.Self:    
        """
        Name of the team.
        """
            
        self._internal.group_name = group_name
    
        return self
    
    def description(self, description: str) -> typing.Self:    
        """
        Role description
        """
            
        self._internal.description = description
    
        return self
    
    def hidden(self, hidden: bool) -> typing.Self:    
        """
        Do not show this role
        """
            
        self._internal.hidden = hidden
    
        return self
    



# Code generated - EDITING IS FUTILE. DO NOT EDIT.

from abc import ABC, abstractmethod
from typing import Generic, TypeVar

T = TypeVar("T")


class Builder(Generic[T], ABC):
    @abstractmethod
    def build(self) -> T:
        pass



# Code generated - EDITING IS FUTILE. DO NOT EDIT.

from ..models import common
import typing
from ..cog import runtime as cogruntime


class Options:
    display_mode: common.BarGaugeDisplayMode
    value_mode: common.BarGaugeValueMode
    name_placement: common.BarGaugeNamePlacement
    show_unfilled: bool
    sizing: common.BarGaugeSizing
    min_viz_width: int
    min_viz_height: int
    reduce_options: common.ReduceDataOptions
    text: typing.Optional[common.VizTextDisplayOptions]
    max_viz_height: int
    orientation: common.VizOrientation

    def __init__(self, display_mode: typing.Optional[common.BarGaugeDisplayMode] = None, value_mode: typing.Optional[common.BarGaugeValueMode] = None, name_placement: typing.Optional[common.BarGaugeNamePlacement] = None, show_unfilled: bool = True, sizing: typing.Optional[common.BarGaugeSizing] = None, min_viz_width: int = 8, min_viz_height: int = 16, reduce_options: typing.Optional[common.ReduceDataOptions] = None, text: typing.Optional[common.VizTextDisplayOptions] = None, max_viz_height: int = 300, orientation: typing.Optional[common.VizOrientation] = None):
        self.display_mode = display_mode if display_mode is not None else common.BarGaugeDisplayMode.GRADIENT
        self.value_mode = value_mode if value_mode is not None else common.BarGaugeValueMode.COLOR
        self.name_placement = name_placement if name_placement is not None else common.BarGaugeNamePlacement.AUTO
        self.show_unfilled = show_unfilled
        self.sizing = sizing if sizing is not None else common.BarGaugeSizing.AUTO
        self.min_viz_width = min_viz_width
        self.min_viz_height = min_viz_height
        self.reduce_options = reduce_options if reduce_options is not None else common.ReduceDataOptions()
        self.text = text
        self.max_viz_height = max_viz_height
        self.orientation = orientation if orientation is not None else common.VizOrientation.AUTO

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "displayMode": self.display_mode,
            "valueMode": self.value_mode,
            "namePlacement": self.name_placement,
            "showUnfilled": self.show_unfilled,
            "sizing": self.sizing,
            "minVizWidth": self.min_viz_width,
            "minVizHeight": self.min_viz_height,
            "reduceOptions": self.reduce_options,
            "maxVizHeight": self.max_viz_height,
            "orientation": self.orientation,
        }
        if self.text is not None:
            payload["text"] = self.text
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "displayMode" in data:
            args["display_mode"] = data["displayMode"]
        if "valueMode" in data:
            args["value_mode"] = data["valueMode"]
        if "namePlacement" in data:
            args["name_placement"] = data["namePlacement"]
        if "showUnfilled" in data:
            args["show_unfilled"] = data["showUnfilled"]
        if "sizing" in data:
            args["sizing"] = data["sizing"]
        if "minVizWidth" in data:
            args["min_viz_width"] = data["minVizWidth"]
        if "minVizHeight" in data:
            args["min_viz_height"] = data["minVizHeight"]
        if "reduceOptions" in data:
            args["reduce_options"] = common.ReduceDataOptions.from_json(data["reduceOptions"])
        if "text" in data:
            args["text"] = common.VizTextDisplayOptions.from_json(data["text"])
        if "maxVizHeight" in data:
            args["max_viz_height"] = data["maxVizHeight"]
        if "orientation" in data:
            args["orientation"] = data["orientation"]        

        return cls(**args)


def variant_config() -> cogruntime.PanelCfgConfig:
    return cogruntime.PanelCfgConfig(
        identifier="bargauge",
        options_from_json_hook=Options.from_json,
        field_config_from_json_hook=None,
    )

